@@ -37,6 +37,8 @@ using Banshee.Base;
 using Banshee.Streaming;
 using Banshee.MediaEngine;
 using Banshee.ServiceStack;
+using Banshee.Configuration;
+using Banshee.Preferences;
 
 namespace Banshee.GStreamer
 {
@@ -54,10 +56,11 @@ namespace Banshee.GStreamer
     internal delegate void BansheePlayerStateChangedCallback (IntPtr player, GstState old_state, GstState new_state, GstState pending_state);
     internal delegate void BansheePlayerIterateCallback (IntPtr player);
     internal delegate void BansheePlayerBufferingCallback (IntPtr player, int buffering_progress);
+    internal delegate void BansheePlayerVisDataCallback (IntPtr player, int channels, int samples, IntPtr data, IntPtr spectrum);
 
     internal delegate void GstTaggerTagFoundCallback (IntPtr player, string tagName, ref GLib.Value value);
     
-    public class PlayerEngine : Banshee.MediaEngine.PlayerEngine, Banshee.MediaEngine.IEqualizer
+    public class PlayerEngine : Banshee.MediaEngine.PlayerEngine, IEqualizer, IVisualizationDataSource
     {
         private uint GST_CORE_ERROR = 0;
         private uint GST_LIBRARY_ERROR = 0;
@@ -71,12 +74,38 @@ namespace Banshee.GStreamer
         private BansheePlayerStateChangedCallback state_changed_callback;
         private BansheePlayerIterateCallback iterate_callback;
         private BansheePlayerBufferingCallback buffering_callback;
+        private BansheePlayerVisDataCallback vis_data_callback;
         private GstTaggerTagFoundCallback tag_found_callback;
         
         private bool buffering_finished;
-        private short pending_volume = -1;
+        private int pending_volume = -1;
         private bool xid_is_set = false;
         
+        private event VisualizationDataHandler data_available = null;
+        public event VisualizationDataHandler DataAvailable {
+            add {
+                if (value == null) {
+                    return;
+                } else if (data_available == null) {
+                    bp_set_vis_data_callback (handle, vis_data_callback);
+                }
+
+                data_available += value;
+            }
+            
+            remove {
+                if (value == null) {
+                    return;
+                }
+
+                data_available -= value;
+
+                if (data_available == null) {
+                    bp_set_vis_data_callback (handle, null);
+                }
+            }
+        }
+        
         public PlayerEngine ()
         {
             if (ServiceManager.IsInitialized) {
@@ -112,6 +141,7 @@ namespace Banshee.GStreamer
             state_changed_callback = new BansheePlayerStateChangedCallback (OnStateChange);
             iterate_callback = new BansheePlayerIterateCallback (OnIterate);
             buffering_callback = new BansheePlayerBufferingCallback (OnBuffering);
+            vis_data_callback = new BansheePlayerVisDataCallback (OnVisualizationData);
             tag_found_callback = new GstTaggerTagFoundCallback (OnTagFound);
             
             bp_set_eos_callback (handle, eos_callback);
@@ -126,10 +156,14 @@ namespace Banshee.GStreamer
             if (pending_volume >= 0) {
                 Volume = (ushort)pending_volume;
             }
+            
+            InstallPreferences ();
+            ReplayGainEnabled = ReplayGainEnabledSchema.Get ();
         }
         
         public override void Dispose ()
         {
+            UninstallPreferences ();
             base.Dispose ();
             bp_destroy (handle);
         }
@@ -230,7 +264,7 @@ namespace Banshee.GStreamer
                 if (CurrentTrack != null) {
                     switch (domain_code) {
                         case GstResourceError.NotFound:
-                            CurrentTrack.PlaybackError = StreamPlaybackError.ResourceNotFound;
+                            CurrentTrack.SavePlaybackError (StreamPlaybackError.ResourceNotFound);
                             break;
                         default:
                             break;
@@ -243,7 +277,7 @@ namespace Banshee.GStreamer
                 if (CurrentTrack != null) {
                     switch (domain_code) {
                         case GstStreamError.CodecNotFound:
-                            CurrentTrack.PlaybackError = StreamPlaybackError.CodecNotFound;
+                            CurrentTrack.SavePlaybackError (StreamPlaybackError.CodecNotFound);
                             break;
                         default:
                             break;
@@ -256,7 +290,7 @@ namespace Banshee.GStreamer
                 if (CurrentTrack != null) {
                     switch (domain_code) {
                         case GstCoreError.MissingPlugin:
-                            CurrentTrack.PlaybackError = StreamPlaybackError.CodecNotFound;
+                            CurrentTrack.SavePlaybackError (StreamPlaybackError.CodecNotFound);
                             break;
                         default:
                             break;
@@ -287,7 +321,35 @@ namespace Banshee.GStreamer
         {
             OnTagFound (ProcessNativeTagResult (tagName, ref value));
         }
+        
+        private void OnVisualizationData (IntPtr player, int channels, int samples, IntPtr data, IntPtr spectrum)
+        {
+            VisualizationDataHandler handler = data_available;
             
+            if (handler == null) {
+                return;
+            }
+            
+            float [] flat = new float[channels * samples];
+            Marshal.Copy (data, flat, 0, flat.Length);
+            
+            float [][] cbd = new float[channels][];
+            for (int i = 0; i < channels; i++) {
+                float [] channel = new float[samples];
+                Array.Copy (flat, i * samples, channel, 0, samples);
+                cbd[i] = channel;
+            }
+            
+            float [] spec = new float[512];
+            Marshal.Copy (spectrum, spec, 0, 512);
+            
+            try {
+                handler (cbd, new float[][] { spec });
+            } catch (Exception e) {
+                Log.Exception ("Uncaught exception during visualization data post.", e);
+            }
+        }
+        
         private static StreamTag ProcessNativeTagResult (string tagName, ref GLib.Value valueRaw)
         {
             if (tagName == String.Empty || tagName == null) {
@@ -314,14 +376,14 @@ namespace Banshee.GStreamer
         }
         
         public override ushort Volume {
-            get { return (ushort)bp_get_volume (handle); }
+            get { return (ushort)Math.Round (bp_get_volume (handle) * 100.0); }
             set { 
                 if ((IntPtr)handle == IntPtr.Zero) {
-                    pending_volume = (short)value;
+                    pending_volume = value;
                     return;
                 }
                 
-                bp_set_volume (handle, (int)value);
+                bp_set_volume (handle, value / 100.0);
                 OnEventChanged (PlayerEvent.Volume);
             }
         }
@@ -329,7 +391,7 @@ namespace Banshee.GStreamer
         public override uint Position {
             get { return (uint)bp_get_position(handle); }
             set { 
-                bp_set_position(handle, (ulong)value);
+                bp_set_position (handle, (ulong)value);
                 OnEventChanged (PlayerEvent.Seek);
             }
         }
@@ -425,6 +487,49 @@ namespace Banshee.GStreamer
             get { return decoder_capabilities; }
         }
         
+        private bool ReplayGainEnabled {
+            get { return bp_replaygain_get_enabled (handle); }
+            set { bp_replaygain_set_enabled (handle, value); }
+        }
+        
+#region Preferences
+
+        private PreferenceBase replaygain_preference;
+
+        private void InstallPreferences ()
+        {
+            PreferenceService service = ServiceManager.Get<PreferenceService> ();
+            if (service == null) {
+                return;
+            }
+            
+            replaygain_preference = service["general"]["misc"].Add (new SchemaPreference<bool> (ReplayGainEnabledSchema, 
+                Catalog.GetString ("_Enable ReplayGain Correction"),
+                Catalog.GetString ("For tracks that have ReplayGain data, automatically scale (normalize) playback volume."),
+                delegate { ReplayGainEnabled = ReplayGainEnabledSchema.Get (); }
+            ));
+        }
+        
+        private void UninstallPreferences ()
+        {
+            PreferenceService service = ServiceManager.Get<PreferenceService> ();
+            if (service == null) {
+                return;
+            }
+            
+            service["general"]["misc"].Remove (replaygain_preference);
+            replaygain_preference = null;
+        }
+        
+        public static readonly SchemaEntry<bool> ReplayGainEnabledSchema = new SchemaEntry<bool> (
+            "player_engine", "replay_gain_enabled", 
+            false,
+            "Enable ReplayGain",
+            "If ReplayGain data is present on tracks when playing, allow volume scaling"
+        );
+
+#endregion
+        
         [DllImport ("libbanshee")]
         private static extern IntPtr bp_new ();
         
@@ -437,6 +542,9 @@ namespace Banshee.GStreamer
         [DllImport ("libbanshee")]
         private static extern void bp_set_error_callback (HandleRef player, BansheePlayerErrorCallback cb);
         
+        [DllImport ("libbanshee")]
+        private static extern void bp_set_vis_data_callback (HandleRef player, BansheePlayerVisDataCallback cb);
+        
         [DllImport ("libbanshee")]
         private static extern void bp_set_state_changed_callback (HandleRef player, 
             BansheePlayerStateChangedCallback cb);
@@ -466,10 +574,10 @@ namespace Banshee.GStreamer
         private static extern void bp_play (HandleRef player);
         
         [DllImport ("libbanshee")]
-        private static extern void bp_set_volume (HandleRef player, int volume);
+        private static extern void bp_set_volume (HandleRef player, double volume);
         
         [DllImport("libbanshee")]
-        private static extern int bp_get_volume (HandleRef player);
+        private static extern double bp_get_volume (HandleRef player);
         
         [DllImport ("libbanshee")]
         private static extern bool bp_can_seek (HandleRef player);
@@ -521,5 +629,11 @@ namespace Banshee.GStreamer
         [DllImport ("libbanshee")]
         private static extern void bp_equalizer_get_frequencies (HandleRef player,
             [MarshalAs (UnmanagedType.LPArray)] out double [] freq);
+            
+        [DllImport ("libbanshee")]
+        private static extern void bp_replaygain_set_enabled (HandleRef player, bool enabled);
+        
+        [DllImport ("libbanshee")]
+        private static extern bool bp_replaygain_get_enabled (HandleRef player);
     }
 }
