@@ -189,13 +189,19 @@ namespace Banshee.Collection.Database
             get {
                 return unfiltered_query ?? unfiltered_query = String.Format (
                     "FROM {0}{1} WHERE {2} {3}",
-                    provider.From, JoinFragment,
+                    From, JoinFragment,
                     String.IsNullOrEmpty (provider.Where) ? "1=1" : provider.Where,
                     ConditionFragment
                 );
             }
         }
 
+        private string from;
+        protected string From {
+            get { return from ?? provider.From; }
+            set { from = value; }
+        }     
+
         public virtual void UpdateUnfilteredAggregates ()
         {
             HyenaSqliteCommand count_command = new HyenaSqliteCommand (String.Format (
@@ -309,7 +315,12 @@ namespace Banshee.Collection.Database
         public override int IndexOf (TrackInfo track)
         {
             lock (this) {
-                return (int) cache.IndexOf (track as DatabaseTrackInfo);
+                if (track is DatabaseTrackInfo) {
+                    return (int) cache.IndexOf (track as DatabaseTrackInfo);
+                } else if (track is Banshee.Streaming.RadioTrackInfo) {
+                    return (int) cache.IndexOf ((track as Banshee.Streaming.RadioTrackInfo).ParentTrack as DatabaseTrackInfo);
+                }
+                return -1;
             }
         }
 
