@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Reflection;
 
 using Mono.Unix;
 
@@ -36,11 +37,14 @@ using Banshee.Sources;
 using Banshee.Collection;
 using Banshee.SmartPlaylist;
 
+using Banshee.Dap.Gui;
+
 namespace Banshee.Dap
 {
-    public abstract class MediaGroupSource : SmartPlaylistSource
+    public abstract class MediaGroupSource : SmartPlaylistSource, IDisposable
     {
         private DapSource parent;
+        private PurchasedMusicActions actions;
         
         public MediaGroupSource (DapSource parent, string name) : base (name, parent)
         {
@@ -51,6 +55,21 @@ namespace Banshee.Dap
             
             Properties.Set<string> ("DeleteTracksFromDriveActionLabel", 
                 String.Format (Catalog.GetString ("Delete From {0}"), parent.Name));
+                
+            if (this is IPurchasedMusicSource) {
+                actions = new PurchasedMusicActions ((IPurchasedMusicSource)this);
+                
+                Properties.Set<Assembly> ("ActiveSourceUIResource.Assembly", Assembly.GetExecutingAssembly ());
+                Properties.SetString ("ActiveSourceUIResource", "PurchasedMusicSourceUI.xml");
+            }
+        }
+        
+        public void Dispose ()
+        {
+            if (actions != null) {
+                actions.Dispose ();
+                actions = null;
+            }
         }
 
         protected override void AfterInitialized ()
@@ -65,14 +84,30 @@ namespace Banshee.Dap
             if (parent != null) {
                 parent.RaiseUpdated ();
             }
+            
+            if (AutoHide) {
+                bool contains_me = parent.ContainsChildSource (this);
+                int count = Count;
+                
+                if (count == 0 && contains_me) {
+                    parent.RemoveChildSource (this);
+                } else if (count > 0 && !contains_me) {
+                    parent.AddChildSource (this);
+                }
+            }
         }
 
-
         /*public override bool AcceptsInputFromSource (Source source)
         {
             return (source is DatabaseSource) && source.Parent != Parent && source != Parent;
         }*/
         
+        private bool auto_hide;
+        public virtual bool AutoHide {
+            get { return auto_hide; }
+            set { auto_hide = value; }
+        }
+        
         public override string ConditionSql {
             get { return base.ConditionSql; }
             protected set { 
