@@ -94,8 +94,9 @@ namespace Banshee.Lastfm.Audioscrobbler
             LastfmCore.AudioscrobblerQueue = queue;
             connection = LastfmCore.Audioscrobbler;
             
-            connection.UpdateNetworkState (NetworkDetect.Instance.Connected);
-            NetworkDetect.Instance.StateChanged += delegate (object o, NetworkStateChangedArgs args) {
+            Network network = ServiceManager.Get<Network> ();
+            connection.UpdateNetworkState (network.Connected);
+            network.StateChanged += delegate (object o, NetworkStateChangedArgs args) {
                 connection.UpdateNetworkState (args.Connected);
             };
             
