@@ -39,6 +39,37 @@ using Mono.Data.SqliteClient;
 
 namespace Hyena.Data.Sqlite
 {
+    public class HyenaDataReader : IDisposable
+    {
+        private IDataReader reader;
+        private bool read = false;
+        
+        public HyenaDataReader (IDataReader reader)
+        {
+            this.reader = reader;
+        }
+        
+        public T Get<T> (int i)
+        {
+            if (!read) {
+                Read ();
+            }
+            return (T) SqliteUtils.FromDbFormat (typeof(T), reader[i]);
+        }
+        
+        public bool Read ()
+        {
+            read = true;
+            return reader.Read ();
+        }
+        
+        public void Dispose ()
+        {
+            reader.Dispose ();
+            reader = null;
+        }
+    }
+    
     public class ExecutingEventArgs : EventArgs
     {
         public readonly SqliteCommand Command;
@@ -120,7 +151,7 @@ namespace Hyena.Data.Sqlite
             Type type = typeof (T);
             using (IDataReader reader = Query (command)) {
                 while (reader.Read ()) {
-                    yield return (T)SqliteUtils.FromDbFormat (type, reader[0]);
+                    yield return (T) SqliteUtils.FromDbFormat (type, reader[0]);
                 }
             }
         }
@@ -207,6 +238,13 @@ namespace Hyena.Data.Sqlite
         // starts a transaction, then another one tries to execute command A (which locks A) but
         // waits for the transaction to finish while holding the lock on A.  If the transaction thread
         // then tries to execute/lock A, it can't.
+        //
+        // I think the way to fix this is to change all the Query/Execute methods above to not lock
+        // the command themselves, but to let that happen in the QueueCommand method (which can be modified
+        // to either take a param_values array and a HyenaCommandType, or have overrides, etc).  That way
+        // the command locking and values applied only when we know the calling thread is not blocking by a
+        // transaction thread.
+        //
         /*public void BeginTransaction ()
         {
             if (transaction_thread == Thread.CurrentThread) {
